# flake8: noqa

from api_fhir_r4.configurations.baseApiFHIRConfiguration import BaseConfiguration
from api_fhir_r4.configurations.baseApiFHIRConfiguration import IdentifierConfiguration
from api_fhir_r4.configurations.baseApiFHIRConfiguration import LocationConfiguration
from api_fhir_r4.configurations.baseApiFHIRConfiguration import MaritalConfiguration
from api_fhir_r4.configurations.baseApiFHIRConfiguration import IssueTypeConfiguration
from api_fhir_r4.configurations.baseApiFHIRConfiguration import ClaimConfiguration
from api_fhir_r4.configurations.baseApiFHIRConfiguration import CoverageEligibilityConfiguration
from api_fhir_r4.configurations.baseApiFHIRConfiguration import CommunicationRequestConfiguration
from api_fhir_r4.configurations.baseApiFHIRConfiguration import CoverageConfiguration
from api_fhir_r4.configurations.baseApiFHIRConfiguration import InvoiceConfiguration
from api_fhir_r4.configurations.baseApiFHIRConfiguration import OrganisationConfiguration
from api_fhir_r4.configurations.baseApiFHIRConfiguration import SubscriptionConfiguration
from api_fhir_r4.configurations.baseApiFHIRConfiguration import PaymentNoticeConfiguration
from api_fhir_r4.configurations.baseApiFHIRConfiguration import BaseApiFhirConfiguration

from api_fhir_r4.configurations.generalConfiguration import GeneralConfiguration
from api_fhir_r4.configurations.R4IdentifierConfig import R4IdentifierConfig
from api_fhir_r4.configurations.R4LocationConfig import R4LocationConfig
from api_fhir_r4.configurations.R4MaritalConfig import R4MaritalConfig
from api_fhir_r4.configurations.R4IssueTypeConfig import R4IssueTypeConfig
from api_fhir_r4.configurations.R4ClaimConfig import R4ClaimConfig
from api_fhir_r4.configurations.R4InvoiceConfig import R4InvoiceConfig
from api_fhir_r4.configurations.R4CoverageEligibilityConfiguration import (
    R4CoverageEligibilityConfiguration,
)
from api_fhir_r4.configurations.R4CommunicationRequestConfig import (
    R4CommunicationRequestConfig,
)
from api_fhir_r4.configurations.R4OrganisationConfig import R4OrganisationConfig
from api_fhir_r4.configurations.R4CoverageConfig import R4CoverageConfig
from api_fhir_r4.configurations.R4SubscriptionConfig import R4SubscriptionConfig
from api_fhir_r4.configurations.R4PaymentNoticeConfig import R4PaymentNoticeConfig

# all specific configurations have to be imported before R4ApiFhirConfig
from api_fhir_r4.configurations.R4ApiFhirConfig import R4ApiFhirConfig
from api_fhir_r4.configurations.moduleConfiguration import ModuleConfiguration
