# Generated by Django 4.2.27 on 2026-01-13 20:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("api_fhir_r4", "0007_alter_historicalsubscription_criteria_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalsubscription",
            name="channel",
            field=models.SmallIntegerField(
                choices=[(1, "rest-hook")], db_column="Channel"
            ),
        ),
        migrations.AlterField(
            model_name="subscription",
            name="channel",
            field=models.SmallIntegerField(
                choices=[(1, "rest-hook")], db_column="Channel"
            ),
        ),
    ]
